def weighted_average(df, values, weights):
    return sum(df[weights] * df[values]) / df[weights].sum()


province_mapping = {
    'a coruña': {'name': 'La Coruña'},
    'albacete': {'name': 'Albacete'},
    'alicante': {'name': 'Alicante'},
    'alicante-alacant': {'name': 'Alicante'},
    'almería': {'name': 'Almería'},
    'almeria': {'name': 'Almería'},
    'araba - álava': {'name': 'Álava'},
    'alava': {'name': 'Álava'},
    'asturias': {'name': 'Asturias'},
    'ávila': {'name': 'Ávila'},
    'avila - avila': {'name': 'Ávila'},
    'avila': {'name': 'Ávila'},
    'badajoz': {'name': 'Badajoz'},
    'balears-illes': {'name': 'Islas Baleares'},
    'barcelona': {'name': 'Barcelona'},
    'bizkaia': {'name': 'Vizcaya'},
    'burgos': {'name': 'Burgos'},
    'cáceres': {'name': 'Cáceres'},
    'caceres': {'name': 'Cáceres'},
    'cádiz': {'name': 'Cádiz'},
    'cadiz': {'name': 'Cádiz'},
    'cantabria': {'name': 'Cantabria'},
    'castellón': {'name': 'Castellón'},
    'castellon-castello': {'name': 'Castellón'},
    'ciudad real': {'name': 'Ciudad Real'},
    'ciudad-real': {'name': 'Ciudad Real'},
    'córdoba': {'name': 'Córdoba'},
    'cordoba': {'name': 'Córdoba'},
    'coruna-a': {'name': 'La Coruña'},
    'cuenca': {'name': 'Cuenca'},
    'gipuzkoa': {'name': 'Guipúzcoa'},
    'guipuzcoa': {'name': 'Guipúzcoa'},
    'girona': {'name': 'Gerona'},
    'granada': {'name': 'Granada'},
    'guadalajara': {'name': 'Guadalajara'},
    'huelva': {'name': 'Huelva'},
    'huesca': {'name': 'Huesca'},
    'illes balears': {'name': 'Islas Baleares'},
    'jaén': {'name': 'Jaén'},
    'jaen': {'name': 'Jaén'},
    'la rioja': {'name': 'La Rioja'},
    'las palmas': {'name': 'Las Palmas'},
    'león': {'name': 'León'},
    'leon': {'name': 'León'},
    'lleida': {'name': 'Lérida'},
    'lugo': {'name': 'Lugo'},
    'madrid': {'name': 'Madrid'},
    'málaga': {'name': 'Málaga'},
    'malaga': {'name': 'Málaga'},
    'mallorca': {'name': 'Islas Baleares'},
    'murcia': {'name': 'Murcia'},
    'navarra': {'name': 'Navarra'},
    'ourense': {'name': 'Orense'},
    'palencia': {'name': 'Palencia'},
    'palmas-las': {'name': 'Las Palmas'},
    'pontevedra': {'name': 'Pontevedra'},
    'rioja-la': {'name': 'La Rioja'},
    'salamanca': {'name': 'Salamanca'},
    'santa cruz de tenerife': {'name': 'Santa Cruz de Tenerife'},
    'santa-cruz-de-tenerife': {'name': 'Santa Cruz de Tenerife'},
    'segovia': {'name': 'Segovia'},
    'sevilla': {'name': 'Sevilla'},
    'soria': {'name': 'Soria'},
    'tarragona': {'name': 'Tarragona'},
    'teruel': {'name': 'Teruel'},
    'toledo': {'name': 'Toledo'},
    'valencia': {'name': 'Valencia'},
    'valencia-valencia': {'name': 'Valencia'},
    'valladolid': {'name': 'Valladolid'},
    'zamora': {'name': 'Zamora'},
    'zaragoza': {'name': 'Zaragoza'},
    'guipúzcoa': {'name': 'Guipúzcoa'},
    'gran canaria': {'name': 'Las Palmas'},
    'tenerife': {'name': 'Santa Cruz de Tenerife'},
    'ibiza - formentera': {'name': 'Islas Baleares'},
    'lanzarote': {'name': 'Las Palmas'},
    'álava': {'name': 'Álava'},
    'palma (la)': {'name': 'Santa Cruz de Tenerife'},
    'vizcaya': {'name': 'Vizcaya'},
    'vizcaya-bizkaia': {'name': 'Vizcaya'},
    'fuerteventura': {'name': 'Las Palmas'},
    'menorca': {'name': 'Islas Baleares'},
    'la coruña': {'name': 'La Coruña'},
    'gomera (la)': {'name': 'Santa Cruz de Tenerife'}
}
column_names = {
    'province': 'province',
    'date': 'version_date',
}
agg_data = {
    'grouping_cols': ['province', 'version_date'],
    'agg_funcs': {
        'number_properties': sum,
        'max_price': max,
        'perc_price_99': max,
        'perc_price_95': max,
        'perc_price_90': max,
        'perc_price_10': min,
        'perc_price_05': min,
        'perc_price_01': min,
        'visits': sum,
        'leads': sum
    }
}
mode_dict = {
    'demand_flat_rent': {'variables': {'is_professional': False, 'leads_only': True, 'operation_type': 'is_rent'},
                         'name_export_file': 'agg_prices_demand_rent.csv'},
    'offer_flat_rent': {'variables': {'is_professional': False, 'leads_only': False, 'operation_type': 'is_rent'},
                        'name_export_file': 'agg_prices_offer_rent.csv'},
    'professionals_flat_rent': {'variables': {'is_professional': True, 'leads_only': True, 'operation_type': 'is_rent'},
                                'name_export_file': 'agg_prices_pro_rent.csv'},
    'demand_houses_rent': {'variables': {'is_professional': False, 'leads_only': True,
                                         'prop_type': 'house', 'operation_type': 'is_rent'},
                           'name_export_file': 'agg_prices_demand_houses_rent.csv'},
    'offer_houses_rent': {'variables': {'is_professional': False, 'leads_only': False,
                                        'operation_type': 'is_rent', 'prop_type': 'house'},
                          'name_export_file': 'agg_prices_demand_houses_rent.csv'},
    'professionals_houses_rent': {'variables': {'is_professional': True, 'leads_only': True,
                                                'operation_type': 'is_rent',  'prop_type': 'house'},
                                  'name_export_file': 'agg_prices_pro_houses_rent.csv'},
    'demand_flat_sell': {'variables': {'is_professional': False, 'leads_only': True},
                         'name_export_file': 'agg_prices_demand_sell.csv'},
    'offer_flat_sell': {'variables': {'is_professional': False, 'leads_only': False},
                        'name_export_file': 'agg_prices_offer_sell.csv'},
    'professionals_flat_sell': {'variables': {'is_professional': True, 'leads_only': True},
                                'name_export_file': 'agg_prices_pro_sell.csv'},
    'demand_houses_sell': {'variables': {'is_professional': False, 'leads_only': True, 'prop_type': 'house'},
                           'name_export_file': 'agg_prices_demand_houses_sell.csv'},
    'offer_houses_sell': {'variables': {'is_professional': False, 'leads_only': False, 'prop_type': 'house'},
                          'name_export_file': 'agg_prices_offer_houses_sell.csv'},
    'professionals_houses_sell': {'variables': {'is_professional': True, 'leads_only': True, 'prop_type': 'house'},
                                  'name_export_file': 'agg_prices_pro_houses_sell.csv'}
}
